/**
 * Copyright 2011 ISO NewEngland Inc.
 *
 * Caveat:
 * This example program is intended to serve only as working example of how to access eMarket MUI Webservices through Java and C# Clients.
 * It is not supported nor intended for production use.
 * Also, it is not intended to serve as a tool for end users.
 *
 * Furthermore, without modification, this example code will not work with eFTR, EES, OutageScheduler,
 * or the Settlements Market System. The example does not implement behavior critical to the interaction with the
 * aforementioned applications including: multi-part MIME, cookie handling, and capturing responses to a file.
 *
 * It is expected that a competent developer of moderate skill and knowledge of Java and/or C# and the HTTP protocol
 * would be able to use these examples as a guide for developing more sophisticated clients to interact
 * with the eMarket MUI WebServices.
 */

package com.areva.sample;

// For reading in properties
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;
import java.util.GregorianCalendar;
import java.util.Date;
import java.util.List;
import java.util.Calendar;

// Security related classes
import java.security.KeyStore;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.TrustManagerFactory;

// For marshaling to XML
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeConstants;

// Web Service related classes
import org.apache.cxf.configuration.jsse.TLSClientParameters;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import org.apache.cxf.jaxws.JaxWsProxyFactoryBean;
import org.apache.cxf.transport.http.HTTPConduit;

// Classes generated from wsdl2java
import com.iso_ne.markets.mui.emkt._0_1.EnergyMarketMUI;
import com.iso_ne.markets.mui.emkt._0_1.MUIFaultException;
import com.iso_ne.markets.mui.emkt.messages.NodeOut;
import com.iso_ne.markets.mui.emkt.messages.GetNode;
import com.iso_ne.markets.mui.emkt.messages.GetNodeResponse;
import com.iso_ne.markets.mui.emkt.messages.PortfolioEntry;
import com.iso_ne.markets.mui.emkt.messages.PortfolioTypeEnum;
import com.iso_ne.markets.mui.emkt.messages.SubmitConfirmation;
import com.iso_ne.markets.mui.emkt.messages.SubmitPortfolio;
import com.iso_ne.markets.mui.emkt.messages.PortfolioInOut;
import com.iso_ne.markets.mui.emkt.messages.TransactionVariance;
import com.iso_ne.markets.mui.emkt.messages.QueryByDayIdAndFuelIndex;
import com.iso_ne.markets.mui.emkt.messages.GetFuelPriceAdjustmentUnit;
import com.iso_ne.markets.mui.emkt.messages.GetFuelPriceAdjustmentUnitResponse;
import com.iso_ne.markets.mui.emkt.messages.FuelPriceAdjustmentUnit;
import com.iso_ne.markets.mui.emkt.messages.TransactionVariance;
import com.iso_ne.markets.mui.emkt.messages.GetSubAccount;
import com.iso_ne.markets.mui.emkt.messages.GetSubAccountResponse;
import com.iso_ne.markets.mui.emkt.messages.GetDemandBid;
import com.iso_ne.markets.mui.emkt.messages.QueryByDayIdBidTypeAndSubAccount;
import com.iso_ne.markets.mui.emkt.messages.QueryBidTypeEnum;
import com.iso_ne.markets.mui.emkt.messages.SubAccountOut;
import com.iso_ne.markets.mui.emkt.messages.GetDemandBidResponse;
import com.iso_ne.markets.mui.emkt.messages.DemandBidInOut;
import com.iso_ne.markets.mui.emkt.messages.DemandBidBySubAccount;





/**
 * This sample program shows how to interact with eMarket Market User
 * Interface (MUI) using web services. The program relies on the having run
 * the eMktMUI.wsdl through the CXF wsdl2java. See the ant build file to see
 * how this is setup.
 */
public class MUIClientSample {

    //-----------------------------------------------------------------
    // Configurable settings are driven from a properties file and held
    // in a local properties object.
    //-----------------------------------------------------------------

    private static String PROPERTIES_PATH = "./MUISample.properties";
	private static Properties properties;

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		properties = getPropertiesFromFile(PROPERTIES_PATH);

		try{

			// Setup the proxy to the web service.
			EnergyMarketMUI webService = createWebServiceClient();

			// Try a simple Query and Submit type operation.
			testGetNode(webService);
			testSubmitPortfolio(webService);
			testFPA(webService);
			testGetSubAccount(webService);
			testGetDemandBid(webService);
		}
		catch(MUIFaultException soapFault){
			System.err.println("\n*** Operation produced a Fault ***\n");
            // display details of soap message
            for (TransactionVariance tv : soapFault.getFaultInfo().getError())
                System.err.printf("ErrorMessage: %s", tv.getReason());
		}
		catch(Exception e){
			System.err.println("\n*** Operation produced a Fault ***\n");
			System.err.println(e.toString());
		}

		System.exit(0);
	}

   /**
    * Executes a "GetNode" operation and prints out the results.
 	* 	@param webService - the service to be used
    */
    private static void testGetNode(EnergyMarketMUI proxy)throws MUIFaultException
    {
        System.out.println("\n-----------------------");
        System.out.println("  Testing GetNode");
        System.out.println("\n-----------------------\n");

        // The XML structure for this message is as follows:
        //    <GetNode party="SomeParty"/>

        // Create the GetNode object. This maps to the <GetNode> element.
		GetNode request = new GetNode();
        request.setParty(properties.getProperty("client.party"));

        // Call the service and print out the response.
		GetNodeResponse response = proxy.getNode(request);
        int nodeCount = response.getNode().size();
        System.out.print("\nNodes returned: " + nodeCount);
        System.out.println(". Here are the first 10:\n");
        int loopCount = 0;
	    for (NodeOut node : response.getNode())
	    {
	        System.out.printf("Node ID = %-8s Name = %-25s Type = %s\n", node.getID(), node.getName(), node.getType());
            loopCount++;
            if (loopCount == 10)
                break;
	    }
    }
    /**
	* Executes a "GetSubAccount" operation and prints out the results.
	* 	@param webService - the service to be used
	*/
	private static void testGetSubAccount(EnergyMarketMUI proxy)throws MUIFaultException
	{
		System.out.println("\n-----------------------");
		System.out.println("  Testing Get Sub Account");
		System.out.println("\n-----------------------\n");

		// The XML structure for this message is as follows:
		//GetSubAccount
		try
		{
			GetSubAccount getSubAccount=new GetSubAccount();
			GregorianCalendar gregorianCalendar = new GregorianCalendar();
			Calendar calendar = Calendar.getInstance();
			calendar.setTime(new Date());
			gregorianCalendar.setTime(calendar.getTime());

			XMLGregorianCalendar xmlGregorianCalendar=DatatypeFactory.newInstance().newXMLGregorianCalendar(gregorianCalendar);
			xmlGregorianCalendar.setTimezone(DatatypeConstants.FIELD_UNDEFINED);

			getSubAccount.setParty(properties.getProperty("client.party"));
			getSubAccount.setDay(xmlGregorianCalendar);

			// Call the service and print out the response.
			GetSubAccountResponse response = proxy.getSubAccount(getSubAccount);

			List<SubAccountOut>  SubAccountOuts= response.getSubAccount();
			System.out.println("\nNodes returned: " + SubAccountOuts.size());
			for (SubAccountOut subAccountOut:SubAccountOuts)
			{
				if(subAccountOut!=null)
				{
					System.out.println("Name : "+subAccountOut.getName()+"\t isDefault: "+subAccountOut.isDefault()+"\t isTerminated: "+subAccountOut.isTerminated());
				}
			}
		}
		catch (DatatypeConfigurationException ex)
		{
			ex.printStackTrace();
		}
    }
    /**
	* Executes a "GetDemandBid" operation and prints out the results.
	* 	@param webService - the service to be used
	*/
	private static void testGetDemandBid(EnergyMarketMUI proxy)throws MUIFaultException
	{
		System.out.println("\n-----------------------");
		System.out.println("  Testing Get Demand Bid");
		System.out.println("\n-----------------------\n");

		// The XML structure for this message is as follows:
		//GetSubAccount
		try
		{

			GregorianCalendar gregorianCalendar = new GregorianCalendar();
			Calendar calendar = Calendar.getInstance();
			calendar.setTime(new Date());
			gregorianCalendar.setTime(calendar.getTime());

			XMLGregorianCalendar xmlGregorianCalendar=DatatypeFactory.newInstance().newXMLGregorianCalendar(gregorianCalendar);
			xmlGregorianCalendar.setTimezone(DatatypeConstants.FIELD_UNDEFINED);

			GetDemandBid getDemandBid=new GetDemandBid();

			QueryByDayIdBidTypeAndSubAccount queryByDayIdBidTypeAndSubAccount=new QueryByDayIdBidTypeAndSubAccount();
			queryByDayIdBidTypeAndSubAccount.setDay(xmlGregorianCalendar);
			queryByDayIdBidTypeAndSubAccount.setBidType(QueryBidTypeEnum.ALL);

			getDemandBid.setQueryFilters(queryByDayIdBidTypeAndSubAccount);
			getDemandBid.setParty(properties.getProperty("client.party"));

			GetDemandBidResponse getDemandBidResponse=proxy.getDemandBid(getDemandBid);
			List<DemandBidInOut> demandBidInOuts=getDemandBidResponse.getDemandBid();
			for (DemandBidInOut demandBidInOut:demandBidInOuts)
			{
				if(demandBidInOut!=null)
				{
					System.out.println("ID : "+demandBidInOut.getID()+"\t Day: "+demandBidInOut.getDay()+"\t BidType: "+demandBidInOut.getBidType());
				}
			}
			List<DemandBidBySubAccount>  demandBidBySubAccounts=getDemandBidResponse.getDemandBidBySubAccount();
			for (DemandBidBySubAccount demandBidBySubAccount:demandBidBySubAccounts)
			{
				if(demandBidBySubAccount!=null)
				{
					demandBidInOuts=demandBidBySubAccount.getDemandBid();
					for (DemandBidInOut demandBidInOut:demandBidInOuts)
					{
						if(demandBidInOut!=null)
						{
							System.out.println("ID : "+demandBidInOut.getID()+"\t Day: "+demandBidInOut.getDay()+"\t BidType: "+demandBidInOut.getBidType()+"\t Sub Account: "+demandBidBySubAccount.getSubAccount());
						}
					}

				}
			}
		}
		catch (DatatypeConfigurationException ex)
		{
			ex.printStackTrace();
		}
    }


    /**
     * Executes a "SubmitPortfolio" operation and prints out the results.
  	* 	@param webService - the service to be used
     */
     private static void testSubmitPortfolio(EnergyMarketMUI proxy)throws MUIFaultException
     {

         System.out.println("\n-----------------------");
         System.out.println("  SubmitPortfolio");
         System.out.println("\n-----------------------\n");

         // The XML structure for this message is as follows:
         // <SubmitPortfolio party="SomeParty">
         //   <!-- 1 or more -->
         //   <Portfolio name="SomeName" type="TheType">
         //     <!-- 1 or more -->
         //     <Resource name="SomeResource" resourceID="TheID"/>
         //   </Portfolio>
         // </SubmitPortfolio>

         // Create the SubmitPortfolio object. This maps to the
         // <SubmitPortfolio> element.
         SubmitPortfolio submitPortfolio = new SubmitPortfolio();
         submitPortfolio.setParty(properties.getProperty("client.party"));

         // Create the portfolio object. This maps to the <Portfolio>
         // element. Assign to the submitPortfolio object.
         PortfolioInOut portfolio = new PortfolioInOut();
         portfolio.setName(properties.getProperty("portfolio.name"));
         portfolio.setType(PortfolioTypeEnum.PNODE);
         submitPortfolio.getPortfolio().add(portfolio);

         // Create the PortfolioEntry object. This maps to the <Resource>
         // element. Assign to the portfolio object.
         PortfolioEntry portfolioEntry = new PortfolioEntry();
         portfolioEntry.setName("LD.STEPSTON23");
         Long id = new Long(16184);
         portfolioEntry.setResourceID(id);
         portfolio.getResource().add(portfolioEntry);

         // Here is the request in XML form
         System.out.println("Calling SubmitPortfolio with this message:\n");
         try
         {
			JAXBContext jc = JAXBContext.newInstance("com.iso_ne.markets.mui.emkt.messages");
			Marshaller m = jc.createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			m.marshal( submitPortfolio, System.out );
         } catch (JAXBException e) {
			e.printStackTrace();
         }

         // Submit the request and print out the response
         SubmitConfirmation response = proxy.submitPortfolio(submitPortfolio);
         System.out.println("Transaction ID = " + response.getTransactionId());


      }
      /**
	       * Executes a "SubmitPortfolio" operation and prints out the results.
	    	* 	@param webService - the service to be used
	       */
	       private static void testFPA(EnergyMarketMUI proxy)throws MUIFaultException
	       {

				System.out.println("\n--------------------------------------------------");
				System.out.println("  Get Fuel Price Adjustment Unit for the given date");
				System.out.println("\n--------------------------------------------------\n");

				// The XML structure for this message is as follows:
				//<GetFuelPriceAdjustmentUnit party="Genon Energy Management_ LLC" xmlns="http://www.markets.iso-ne.com/MUI/eMkt/Messages">
				//  <QueryFilters>
				//	<Day>2014-07-08</Day>
				//	<FuelIndex />
				//  </QueryFilters>
				//	</GetFuelPriceAdjustmentUnit>

				// Create the FPA object. This maps to the
				// <GetFuelPriceAdjustmentUnit> element.
				try
				{
					GregorianCalendar gregorianCalendar = new GregorianCalendar();
					Calendar calendar = Calendar.getInstance();
					calendar.setTime(new Date());
					calendar.add(Calendar.DATE, 1);

					gregorianCalendar.setTime(calendar.getTime());

					XMLGregorianCalendar xmlGregorianCalendar=DatatypeFactory.newInstance().newXMLGregorianCalendar(gregorianCalendar);
					xmlGregorianCalendar.setTimezone(DatatypeConstants.FIELD_UNDEFINED);

					QueryByDayIdAndFuelIndex  queryByDayIdAndFuelIndex=new QueryByDayIdAndFuelIndex();

					queryByDayIdAndFuelIndex.setDay(xmlGregorianCalendar);
					GetFuelPriceAdjustmentUnit getFuelPriceAdjustmentUnit=new GetFuelPriceAdjustmentUnit();
					getFuelPriceAdjustmentUnit.setQueryFilters(queryByDayIdAndFuelIndex);
					getFuelPriceAdjustmentUnit.setParty(properties.getProperty("client.party"));

					// Call the service and print out the response.
					GetFuelPriceAdjustmentUnitResponse getFuelPriceAdjustmentUnitResponse = proxy.getFuelPriceAdjustmentUnit(getFuelPriceAdjustmentUnit);
					int FuelPriceAdjustmentUnitCount = getFuelPriceAdjustmentUnitResponse.getFuelPriceAdjustmentUnit().size();
					System.out.print("\n FuelPriceAdjustmentUnit Returned: " + FuelPriceAdjustmentUnitCount);
					System.out.println(". Here are the first 10:\n");
					int loopCount = 0;
					for (FuelPriceAdjustmentUnit fuelPriceAdjustmentUnit : getFuelPriceAdjustmentUnitResponse.getFuelPriceAdjustmentUnit())
					{
						System.out.printf("Unit Long Name = %-25s Day = %-25s ID = %s\n", fuelPriceAdjustmentUnit.getUnitLongName(), fuelPriceAdjustmentUnit.getDay(), fuelPriceAdjustmentUnit.getID());
						loopCount++;
						if (loopCount == 10)
							break;
					}
				}
				catch (DatatypeConfigurationException ex)
				{
					ex.printStackTrace();
				}

      }

 	/**	Create the CXF webservice Proxy.
	 * 	@return webService used throughout the project
	 */
	private static EnergyMarketMUI createWebServiceClient(){
		// factory for creating interface
		JaxWsProxyFactoryBean factory = new JaxWsProxyFactoryBean();

		// logging for in and out. Add these if you want more info spit out
		factory.getInInterceptors().add(new LoggingInInterceptor());
		factory.getOutInterceptors().add(new LoggingOutInterceptor());

		// add interface and end point
		factory.setServiceClass(EnergyMarketMUI.class);
		factory.setAddress(properties.getProperty("endpoint.link"));

		// instance of our web service message options
		EnergyMarketMUI webService = (EnergyMarketMUI) factory.create();

		// add certificate connectivity properties
		configureSSLOnTheClient(webService);

		return webService;
	}

	/** This method configures our transport layer to use <i>two way</i> secure HTTP
	 * 	To do this we're configuring the certificates we trust as well as the certificate
	 * 	that we are.
	 * 	@param c - the CXF Proxy that we want to wrap with SSL
	 */
	private static void configureSSLOnTheClient(Object c){
		// end point obtained
		org.apache.cxf.endpoint.Client client = ClientProxy.getClient(c);

		// conduit we're using to transfer data
		HTTPConduit httpConduit = (HTTPConduit)client.getConduit();

		try{
			// holds parameters for configuration
			TLSClientParameters tlsParams = new TLSClientParameters();
			tlsParams.setDisableCNCheck(true);

			// format type for server side certificates
			KeyStore trustStore = KeyStore.getInstance(
				properties.getProperty("srvr.trust.fmt")
			);

			// location of server trusted list of certificates
			File serverTrusts = new File(
				properties.getProperty("srvr.trust.kstr")
			);

			// add trusted server certificates to store
            String propValue = properties.getProperty("srvr.trust.pswd");
			trustStore.load(new FileInputStream(serverTrusts),
				propValue.toCharArray()
			);

			// trust manager generated
			TrustManagerFactory trustFactory =
				TrustManagerFactory.getInstance(
						TrustManagerFactory.getDefaultAlgorithm()
				);// uses default Algorithm

			// initialize trust manager with the file acquired
			trustFactory.init(trustStore);

			// this configures our client to trust these certificates
			tlsParams.setTrustManagers(trustFactory.getTrustManagers());

			// keys for client side
			KeyStore keyStore = KeyStore.getInstance(
				properties.getProperty("client.key.fmt")
			);

			// location of client key
			File clientKey = new File(
				properties.getProperty("client.key.file")
			);

			// load certificate file into key store using password
            propValue = properties.getProperty("client.key.pswd");
			keyStore.load(new FileInputStream(clientKey),
				propValue.toCharArray()
			);

			// manager for keys
			KeyManagerFactory keyFactory =
				KeyManagerFactory.getInstance(
						KeyManagerFactory.getDefaultAlgorithm()
				);// uses default Algorithm

			// authorize use of key
            propValue = properties.getProperty("client.key.pswd");
			keyFactory.init(keyStore,
				propValue.toCharArray()
			);

			// this sets our client to use the configured key and certificate as our credentials
			tlsParams.setKeyManagers(keyFactory.getKeyManagers());

			// add parameters to conduit
			httpConduit.setTlsClientParameters(tlsParams);

		// exceptions are caught and dealt with the same way
		} catch (Exception e) {
            System.out.println("Security configuration failed with the following: ");
            e.printStackTrace();
            System.out.println("-------");
        }// end catch
	}//end


	/** Retrieves properties from a file for the use of this program.
	 * 	@param path - a path to the properties file
	 * 	@return properties - the properties to be used
	 */
	private static Properties getPropertiesFromFile(String path){
		Properties properties = new Properties();
		File file = new File(path);

		// attempt to read the file and absorb the properties contained within
		try {
			FileInputStream fileReader = new FileInputStream(file);
			properties.load(fileReader);
		} catch (FileNotFoundException e) {
			// print errors
			e.printStackTrace();
			System.exit(1);
		}catch (IOException e) {
			// print errors
			e.printStackTrace();
			System.exit(1);
		}

		return properties;
	}



}// end class MUIClientSample
